package  
{
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.utils.Input;
	import net.flashpunk.Sfx;
	
	/**
	 * ...
	 * @author Benn
	 */
	public class PlayerBall extends Entity 
	{

		[Embed(source = "../assets/audio/punt.mp3")] private static const PUNT:Class;
		[Embed(source = "../assets/audio/wallbounce.mp3")] private static const WALLBOUNCE : Class;
		[Embed(source = "../assets/audio/fire.mp3")] private static const FIRE:Class;
		[Embed(source = "../assets/audio/squarehit.mp3")] private static const SQUAREHIT:Class;
		
		private var FireSfx:Sfx = new Sfx(FIRE);
		private var PuntSfx:Sfx = new Sfx(PUNT);
		private var WallbounceSfx:Sfx = new Sfx(WALLBOUNCE);
		private var SquareHitSfx:Sfx = new Sfx(SQUAREHIT);
		
		private var CurrentLaunchStage : int = 0; //0 - unlaunched, 1 - punted, 2 - freemove
		private var velocityX : Number = 0;
		private var velocityY : Number = 0;
		private var startPosX : int;
		private var startPosY : int;
		
		private var Gravity : Number = 2000;
		private var PuntStrength : Number = 1300;
		private var FireStrength : Number = 1100;
		
		private var CanDoubleJump : Boolean = false;
		
		public function PlayerBall(posx: int, posy: int) 
		{
			startPosX = posx;
			startPosY = posy;
			x = startPosX;
			y = startPosY;
			setHitbox(16, 16, 12, 12);
			type = "ball";
		}
		
		public function GetCanDoubleJump():Boolean
		{
			return CanDoubleJump;
		}
		
		public function GetState():int
		{
			return CurrentLaunchStage;
		}
		
		private function UpdateFreeMove():void
		{
			
			x += velocityX * FP.elapsed;
			
			if (collide("world", x, y))
			{
				WallbounceSfx.play();
				x -= velocityX * FP.elapsed;
				velocityX = -velocityX;
			}
			
			velocityY += Gravity * FP.elapsed;
			
			y += velocityY * FP.elapsed;
			
			if (collide("world", x, y))
			{
				WallbounceSfx.play();
				y -= velocityY * FP.elapsed;
				velocityY = -velocityY;
			}
			
			if (y > 500 || x < -10 || x > 650)
			{
				Reset();
			}
			
			var Goal : SquareGoal = collide("goal", x, y) as SquareGoal;
			
			if (Goal)
			{
				if (Goal.GetDead() == false)
				{
					SquareHitSfx.play();
					Goal.Die();
				}
			}
			
			var DoubleJump : SquareDoubleJump = collide("doublejump", x, y) as SquareDoubleJump;
			
			if (DoubleJump)
			{
				if (DoubleJump.GetDead() == false)
				{
					SquareHitSfx.play();
					CanDoubleJump = true;
					DoubleJump.Die();
				}
			}
			
		}
		
		private function UpdatePunt():void
		{
			velocityY += Gravity * FP.elapsed;
			
			y += velocityY * FP.elapsed;
			
			if (collide("world", x, y))
			{
				CurrentLaunchStage = 0;
				x = startPosX;
				y = startPosY;
			}
		}
		
		public function Punt(strength : Number):void
		{
			PuntSfx.play();
			velocityY = 0 - ((0.5 + (strength / 2)) * PuntStrength);
			CurrentLaunchStage = 1;
		}
		
		public function Fire(strength : Number):void
		{
			
			FireSfx.play();
			
			var directionX : Number = (FP.camera.x + Input.mouseX) - x;
			var directionY : Number = (FP.camera.y + Input.mouseY) - y;
		
			//normalize
			var dirDistance : Number = FP.distance(x, y, FP.camera.x + Input.mouseX, FP.camera.y + Input.mouseY)
			
			directionX /= dirDistance;
			directionY /= dirDistance;
			
			velocityX = directionX * strength * FireStrength;
			velocityY = directionY * strength * FireStrength;
			
			CurrentLaunchStage = 2;
			
			CanDoubleJump = false;

		}
		
		public function Reset():void
		{
			velocityX = 0;
			velocityY = 0;
			x = startPosX;
			y = startPosY;
			CurrentLaunchStage = 0;
		}
		
		override public function update():void 
		{
			super.update();
			
			if (CurrentLaunchStage == 1)
			{
				UpdatePunt();
			}
			else if (CurrentLaunchStage == 2)
			{
				UpdateFreeMove();
			}
			
		}
		
	}

}